#############################################Packages required########################################
library(deSolve)
library(ggplot2)
library(reshape2)
library(plotly)
library(dplyr)
library(tikzDevice)
library(pracma)
library(cowplot)
library(patchwork)

###################################SIR(S) model without seasonality###################################
sir_s_.model=function(t,x,parameters){
  SP=x[1]
  IP=x[2]
  R=x[3]
  SS=x[4]
  IS=x[5]
  N=parameters["N"]
  mu=parameters["mu"]
  delta=parameters["delta"]
  gamma=parameters["gamma"]
  beta=parameters["beta"]
  alpha=parameters["alpha"]
  epsilon=parameters["epsilon"]
  dSPdt=mu*N-(beta*SP*(IP+alpha*IS)/N)-mu*SP
  dIPdt=(beta*SP*(IP+alpha*IS)/N)-(gamma+mu)*IP
  dRdt=gamma*(IP+IS)-(delta+mu)*R
  dSSdt=delta*R-epsilon*(beta*SS*(IP+alpha*IS)/N)-mu*SS
  dISdt=epsilon*(beta*SS*(IP+alpha*IS)/N)-(gamma+mu)*IS
  dxdt <- c(dSPdt,dIPdt,dRdt,dSSdt,dISdt)
  list(dxdt)
}

#############################Plots for SIR(S)  model without seasonality##############################
#####This function (and other plotting functions below) returns two plots, one 
#####with infectious fractions curves,and the other, with the three immune statuses
plots.no.seasonality=function(alpha,titles){
  epsilonVector=c(0.4,0.5,0.6,0.7,0.8,0.9,1)
  valuesIP=list()
  valuesIS=list()
  valuesR=list()
  valuesSP=list()
  valuesSS=list()
  R0.listOrig=read.table("NY.csv",header=T,sep=",")
  R0.list=R0.listOrig$hku1
  delta=1/(1*365)
  N=1
  mu=1/(50*365)
  gamma=1/5
  beta=mean(R0.list)/5
  I0=1e-9
  xstart=c(SP=1-I0,IP=I0,R=0,SS=0,IS=0) 
  times=seq(from=0,to=5*365,by=1)
  for  (i in  1:length(epsilonVector)){
    epsilon=epsilonVector[i]
    parameters=c(N=N,mu=mu,gamma=gamma,beta=beta,alpha=alpha,
                 epsilon=epsilon,delta=delta)
    out<-as.data.frame(rk(func=sir_s_.model,y=xstart,times=times,parms=parameters))
    valuesIP[[i]]=out$IP
    valuesIS[[i]]=out$IS
    valuesR[[i]]=out$R
    valuesSS[[i]]=out$SS
    valuesSP[[i]]=out$SP
  }
  dataValuesIP=t(do.call(cbind,valuesIP))
  dataValuesIS=t(do.call(cbind,valuesIS))
  dataValuesSS=t(do.call(cbind,valuesSS))
  dataValuesSP=t(do.call(cbind,valuesSP))
  dataValuesR=t(do.call(cbind,valuesR))
  valuesMeltIP=melt(dataValuesIP)
  valuesMeltIS=melt(dataValuesIS)
  valuesMeltSS=melt(dataValuesSS)
  valuesMeltSP=melt(dataValuesSP)
  valuesMeltR=melt(dataValuesR)
  for (i in 1:length(epsilonVector)){
    valuesMeltIP$Var1[valuesMeltIP$Var1==i]=epsilonVector[i]
    valuesMeltIS$Var1[valuesMeltIS$Var1==i]=epsilonVector[i]
    valuesMeltSS$Var1[valuesMeltSS$Var1==i]=epsilonVector[i]
    valuesMeltSP$Var1[valuesMeltSP$Var1==i]=epsilonVector[i]
    valuesMeltR$Var1[valuesMeltR$Var1==i]=epsilonVector[i]
  }
  valuesMeltIP$Var1=as.character(valuesMeltIP$Var1)
  valuesMeltIS$Var1=as.character(valuesMeltIS$Var1)
  valuesMeltSS$Var1=as.character(valuesMeltSS$Var1)
  valuesMeltSP$Var1=as.character(valuesMeltSP$Var1)
  valuesMeltR$Var1=as.character(valuesMeltR$Var1)
  
  scene=list(camera=list(eye=list(x=-1.5,y=-1.5,z=-1.5)),
             yaxis=list(title="ϵ",titlefont=list(size=25),tickfont=list(size=17),ticksuffix = "   ",automargin=T),
             zaxis=list(title="Infections",titlefont=list(size=25),tickfont=list(size=17),ticksuffix = "   ",automargin=T),
             xaxis=list(title="Time (years)",titlefont=list(size=25),tickfont=list(size=17),ticksuffix = "   ",automargin=T))
  IP.IS=bind_rows(valuesMeltIP,valuesMeltIS,.id=c("infection"))
  IP.IS$infection[IP.IS$infection=="1"]="solid"
  IP.IS$infection[IP.IS$infection=="2"]="dashdot"
  plotCombined=plot_ly(data=IP.IS,y=~Var1,x=~Var2/365,z=~value,type='scatter3d',
                       mode='lines',color=~Var1,
                       colors=c("chartreuse4","darkgoldenrod1","tan2","red",'cornsilk4',"darkorchid1","blue"),
                       linetype=~I(infection),line=list(width=6))%>% 
    layout(title=titles[1],scene=scene,showlegend=F)
  
  
  scene=list(camera=list(eye=list(x=-1.5,y=-1.5,z=-1.5)),
             yaxis=list(title="ϵ",titlefont=list(size=25),tickfont=list(size=17),ticksuffix = "   ",automargin=T),
             zaxis=list(title="Fraction of pop.",titlefont=list(size=25),tickfont=list(size=17),ticksuffix = "   ",automargin=T),
             xaxis=list(title="Time (years)",titlefont=list(size=25),tickfont=list(size=17),ticksuffix = "   ",automargin=T))
  SP.R.SS=bind_rows(valuesMeltSP,valuesMeltR,valuesMeltSS,.id=c("immunity"))
  SP.R.SS=SP.R.SS[SP.R.SS$Var1 %in% c(0.4,0.7,1),]
  plotSusc=plot_ly(data=SP.R.SS,y=~Var1,x=~Var2/365,z=~value,type='scatter3d',
                   mode='lines',color=~Var1,
                   colors=c("chartreuse4","red","blue"),
                   linetype=~immunity,line=list(width=6))%>% 
    layout(title=titles[2],scene=scene,showlegend=F)
  plots=list(plotCombined,plotSusc)
  return(plots)
}

titles1=c("alpha=1, no seasonality, infections","alpha=1, no seasonality, immunity")
alpha1plots=plots.no.seasonality(1,titles1)
alpha1plots[[1]]
alpha1plots[[2]]

titles2=c("alpha=0.8, no seasonality, infections","alpha=1, no seasonality, immunity")
alpha0.8plots=plots.no.seasonality(0.8,titles2)
alpha0.8plots[[1]]
alpha0.8plots[[2]]

#####################################SIR(S) model with seasonality####################################
sir_s_.model.seasonality=function(t,x,parameters){
  SP=x[1]
  IP=x[2]
  R=x[3]
  SS=x[4]
  IS=x[5]
  N=parameters[["N"]]
  mu=parameters[["mu"]]
  delta=parameters[["delta"]]
  gamma=parameters[["gamma"]]
  R0.list=parameters[["R0.list"]]
  alpha=parameters[["alpha"]]
  epsilon=parameters[["epsilon"]]
  beta=R0.list[t]*gamma
  dSPdt=mu*N-(beta*SP*(IP+alpha*IS)/N)-mu*SP
  dIPdt=(beta*SP*(IP+alpha*IS)/N)-(gamma+mu)*IP
  dRdt=gamma*(IP+IS)-(delta+mu)*R
  dSSdt=delta*R-epsilon*(beta*SS*(IP+alpha*IS)/N)-mu*SS
  dISdt=epsilon*(beta*SS*(IP+alpha*IS)/N)-(gamma+mu)*IS
  dxdt <- c(dSPdt,dIPdt,dRdt,dSSdt,dISdt)
  list(dxdt)
}

###############################Plots for SIR(S)  model with seasonality###############################
plots.seasonality=function(alpha,shift,R0.listOrig,titles){
  epsilonVector=c(0.4,0.5,0.6,0.7,0.8,0.9,1)
  valuesIP=list()
  valuesIS=list()
  valuesR=list()
  valuesSP=list()
  valuesSS=list()
  gamma=7/5
  delta=1/(1*52)
  N=1
  mu=1/(50*52)
  times=seq(from=1,to=5*52,by=1)
  R0.list.a=R0.listOrig$hku1
  R0.list.shifted=circshift(R0.list.a,-shift)
  R0.list=rep(R0.list.shifted,length=length(times))
  I0=1e-9
  xstart=c(SP=1-I0,IP=I0,R=0,SS=0,IS=0) 
  for  (i in  1:length(epsilonVector)){
    epsilon=epsilonVector[i]
    parameters=list(N=N,mu=mu,gamma=gamma,alpha=alpha,
                    epsilon=epsilon,delta=delta,R0.list=R0.list)
    out=as.data.frame(rk(func=sir_s_.model.seasonality,y=xstart,times=times,parms=parameters))
    valuesIP[[i]]=out$IP
    valuesIS[[i]]=out$IS
    valuesR[[i]]=out$R
    valuesSS[[i]]=out$SS
    valuesSP[[i]]=out$SP
  }
  dataValuesIP=t(do.call(cbind,valuesIP))
  dataValuesIS=t(do.call(cbind,valuesIS))
  dataValuesSS=t(do.call(cbind,valuesSS))
  dataValuesSP=t(do.call(cbind,valuesSP))
  dataValuesR=t(do.call(cbind,valuesR))
  valuesMeltIP=melt(dataValuesIP)
  valuesMeltIS=melt(dataValuesIS)
  valuesMeltSS=melt(dataValuesSS)
  valuesMeltSP=melt(dataValuesSP)
  valuesMeltR=melt(dataValuesR)
  for (i in 1:length(epsilonVector)){
    valuesMeltIP$Var1[valuesMeltIP$Var1==i]=epsilonVector[i]
    valuesMeltIS$Var1[valuesMeltIS$Var1==i]=epsilonVector[i]
    valuesMeltSS$Var1[valuesMeltSS$Var1==i]=epsilonVector[i]
    valuesMeltSP$Var1[valuesMeltSP$Var1==i]=epsilonVector[i]
    valuesMeltR$Var1[valuesMeltR$Var1==i]=epsilonVector[i]
  }
  valuesMeltIP$Var1=as.character(valuesMeltIP$Var1)
  valuesMeltIS$Var1=as.character(valuesMeltIS$Var1)
  valuesMeltSS$Var1=as.character(valuesMeltSS$Var1)
  valuesMeltSP$Var1=as.character(valuesMeltSP$Var1)
  valuesMeltR$Var1=as.character(valuesMeltR$Var1)
  
  scene=list(camera=list(eye=list(x=-1.5,y=-1.5,z=-1.5)),
             yaxis=list(title="ϵ",titlefont=list(size=25),tickfont=list(size=17),ticksuffix = "   ",automargin=T),
             zaxis=list(title="Infections",titlefont=list(size=25),tickfont=list(size=17),ticksuffix = "   ",automargin=T),
             xaxis=list(title="Time (years)",titlefont=list(size=25),tickfont=list(size=17),ticksuffix = "   ",automargin=T))
  IP.IS=bind_rows(valuesMeltIP,valuesMeltIS,.id=c("infection"))
  IP.IS$infection[IP.IS$infection=="1"]="solid"
  IP.IS$infection[IP.IS$infection=="2"]="dashdot"
  plotCombined=plot_ly(data=IP.IS,y=~Var1,x=~Var2/52,z=~value,type='scatter3d',
                       mode='lines',color=~Var1,
                       colors=c("chartreuse4","darkgoldenrod1","tan2","red",'cornsilk4',"darkorchid1","blue"),
                       linetype=~I(infection),line=list(width=6))%>% 
    layout(title=titles[1],scene=scene,showlegend=F)
  
  scene=list(camera=list(eye=list(x=-1.5,y=-1.5,z=-1.5)),
             yaxis=list(title="ϵ",titlefont=list(size=25),tickfont=list(size=17),ticksuffix = "   ",automargin=T),
             zaxis=list(title="Fraction of pop.",titlefont=list(size=25),tickfont=list(size=17),ticksuffix = "   ",automargin=T),
             xaxis=list(title="Time (years)",titlefont=list(size=25),tickfont=list(size=17),ticksuffix = "   ",automargin=T))
  SP.R.SS=bind_rows(valuesMeltSP,valuesMeltR,valuesMeltSS,.id=c("immunity"))
  SP.R.SS=SP.R.SS[SP.R.SS$Var1 %in% c(0.4,0.7,1),]
  plotSusc=plot_ly(data=SP.R.SS,y=~Var1,x=~Var2/52,z=~value,type='scatter3d',
                   mode='lines',color=~Var1,
                   colors=c("chartreuse4","red","blue"),
                   linetype=~immunity,line=list(width=6))%>% 
    layout(title=titles[2],scene=scene,showlegend=F)
  plots=list(plotCombined,plotSusc)
}

R0.listOrig=read.table("NY.csv",header=T,sep=",")

titles3=c("seasonality, no lag, infections","seasonality, no lag, immunity")
seasonal.no.lag=plots.seasonality(1,0,R0.listOrig,titles3)
seasonal.no.lag[[1]]
seasonal.no.lag[[2]]

titles4=c("seasonality, 13wk lag, infections","seasonality, 13wk lag, immunity")
seasonal.13weeks.lag=plots.seasonality(1,12,R0.listOrig,titles4)
seasonal.13weeks.lag[[1]]
seasonal.13weeks.lag[[2]]

titles5=c("seasonality, 26wk lag, infections","seasonality, 26wk lag, immunity")
seasonal.26weeks.lag=plots.seasonality(1,25,R0.listOrig,titles5)
seasonal.26weeks.lag[[1]]
seasonal.26weeks.lag[[2]]

titles6=c("seasonality, 39wk lag, infections","seasonality, 39wk lag, immunity")
seasonal.39weeks.lag=plots.seasonality(1,38,R0.listOrig,titles6)
seasonal.39weeks.lag[[1]]
seasonal.39weeks.lag[[2]]

###########################################With Jakarta R0############################################
R0.listOrig=read.table("Jakarta.csv",header=T,sep=",")

titles7=c("seasonality, jakarta, infections","seasonality, jakarta, immunity")
seasonal.no.lag.jakarta=plots.seasonality(1,0,R0.listOrig,titles7)
seasonal.no.lag.jakarta[[1]]
seasonal.no.lag.jakarta[[2]]

############################################With Delhi R0#############################################
R0.listOrig=read.table("Delhi.csv",header=T,sep=",")

titles8=c("seasonality, delhi, infections","seasonality, delhi, immunity")
seasonal.no.lag.delhi=plots.seasonality(1,0,R0.listOrig,titles8)
seasonal.no.lag.delhi[[1]]
seasonal.no.lag.delhi[[2]]

##########################SIR(S) model with seasonality and social distancing#########################
sir_s_.model.seasonality.socdis=function(t,x,parameters){
  
  SP=x[1]
  IP=x[2]
  R=x[3]
  SS=x[4]
  IS=x[5]
  
  N=parameters[["N"]]
  mu=parameters[["mu"]]
  delta=parameters[["delta"]]
  gamma=parameters[["gamma"]]
  R0.list=parameters[["R0.list"]]
  alpha=parameters[["alpha"]]
  epsilon=parameters[["epsilon"]]
  beta=R0.list[t]*gamma
  #Social distancing
  R0red=parameters[["R0red"]]
  tred1=parameters[["tred1"]]
  tred2=parameters[["tred2"]]
  tred3=parameters[["tred3"]]
  tred4=parameters[["tred4"]]
  
  if((t>tred1) & (t<tred2)){
    beta = beta * R0red
  }
  if((t>tred3) & (t<tred4)){
    beta = beta * R0red
  }
  
  dSPdt=mu*N-(beta*SP*(IP+alpha*IS)/N)-mu*SP
  dIPdt=(beta*SP*(IP+alpha*IS)/N)-(gamma+mu)*IP
  dRdt=gamma*(IP+IS)-(delta+mu)*R
  dSSdt=delta*R-epsilon*(beta*SS*(IP+alpha*IS)/N)-mu*SS
  dISdt=epsilon*(beta*SS*(IP+alpha*IS)/N)-(gamma+mu)*IS
  
  dxdt <- c(dSPdt,dIPdt,dRdt,dSSdt,dISdt)
  list(dxdt)
}


#####################Plots for SIR(S)  model with seasonality and social distancing###################
#####The id variable is to distinguish when to plot the extra plots, panels D and E
#####of Fig. 3. If id=TRUE, latex code will be generated for these plots, which can
#####then be compiled to produce the panels.
plots.soc.dis.seasonality=function(alpha,tred1,tred2,tred3,tred4,id,titles){
  epsilonVector=c(0.4,0.5,0.6,0.7,0.8,0.9,1)
  valuesIP=list()
  valuesIS=list()
  valuesR=list()
  valuesSP=list()
  valuesSS=list()
  delta=1/(1*52)
  N=1
  mu=1/(50*52)
  gamma=7/5
  R0red=0.6
  times=seq(from=1,to=5*52,by=1)
  R0.listOrig=read.table("NY.csv",header=T,sep=",")
  R0.list.a=R0.listOrig$hku1
  R0.list=rep(R0.list.a,length=length(times))
  I0=1e-9
  xstart=c(SP=1-I0,IP=I0,R=0,SS=0,IS=0) 
  for  (i in  1:length(epsilonVector)){
    epsilon=epsilonVector[i]
    parameters=list(N=N,mu=mu,gamma=gamma,alpha=alpha,
                    epsilon=epsilon,delta=delta,
                    R0.list=R0.list,R0red=R0red,tred1=tred1,
                    tred2=tred2,tred3=tred3,tred4=tred4)
    out=as.data.frame(rk(func=sir_s_.model.seasonality.socdis,y=xstart,times=times,parms=parameters))
    valuesIP[[i]]=out$IP
    valuesIS[[i]]=out$IS
    valuesR[[i]]=out$R
    valuesSS[[i]]=out$SS
    valuesSP[[i]]=out$SP
  }
  dataValuesIP=t(do.call(cbind,valuesIP))
  dataValuesIS=t(do.call(cbind,valuesIS))
  dataValuesSS=t(do.call(cbind,valuesSS))
  dataValuesSP=t(do.call(cbind,valuesSP))
  dataValuesR=t(do.call(cbind,valuesR))
  valuesMeltIP=melt(dataValuesIP)
  valuesMeltIS=melt(dataValuesIS)
  valuesMeltSS=melt(dataValuesSS)
  valuesMeltSP=melt(dataValuesSP)
  valuesMeltR=melt(dataValuesR)
  for (i in 1:length(epsilonVector)){
    valuesMeltIP$Var1[valuesMeltIP$Var1==i]=epsilonVector[i]
    valuesMeltIS$Var1[valuesMeltIS$Var1==i]=epsilonVector[i]
    valuesMeltSS$Var1[valuesMeltSS$Var1==i]=epsilonVector[i]
    valuesMeltSP$Var1[valuesMeltSP$Var1==i]=epsilonVector[i]
    valuesMeltR$Var1[valuesMeltR$Var1==i]=epsilonVector[i]
  }
  valuesMeltIP$Var1=as.character(valuesMeltIP$Var1)
  valuesMeltIS$Var1=as.character(valuesMeltIS$Var1)
  valuesMeltSS$Var1=as.character(valuesMeltSS$Var1)
  valuesMeltSP$Var1=as.character(valuesMeltSP$Var1)
  valuesMeltR$Var1=as.character(valuesMeltR$Var1)
  
  scene=list(camera=list(eye=list(x=-1.5,y=-1.5,z=-1.5)),
             yaxis=list(title="ϵ",titlefont=list(size=25),tickfont=list(size=17),ticksuffix = "   ",automargin=T),
             zaxis=list(title="Infections",titlefont=list(size=25),tickfont=list(size=17),ticksuffix = "   ",automargin=T),
             xaxis=list(title="Time (years)",titlefont=list(size=25),tickfont=list(size=17),ticksuffix = "   ",automargin=T))
  IP.IS=bind_rows(valuesMeltIP,valuesMeltIS,.id=c("infection"))
  IP.IS$infection[IP.IS$infection=="1"]="solid"
  IP.IS$infection[IP.IS$infection=="2"]="dashdot"
  plotCombined=plot_ly(data=IP.IS,y=~Var1,x=~Var2/52,z=~value,type='scatter3d',
                       mode='lines',color=~Var1,
                       colors=c("chartreuse4","darkgoldenrod1","tan2","red",'cornsilk4',"darkorchid1","blue"),
                       linetype=~I(infection),line=list(width=6))%>% 
    layout(title=titles[1],scene=scene,showlegend=F)
  
  scene=list(camera=list(eye=list(x=-1.5,y=-1.5,z=-1.5)),
             yaxis=list(title="ϵ",titlefont=list(size=25),tickfont=list(size=17),ticksuffix = "   ",automargin=T),
             zaxis=list(title="Fraction of pop.",titlefont=list(size=25),tickfont=list(size=17),ticksuffix = "   ",automargin=T),
             xaxis=list(title="Time (years)",titlefont=list(size=25),tickfont=list(size=17),ticksuffix = "   ",automargin=T))
  SP.R.SS=bind_rows(valuesMeltSP,valuesMeltR,valuesMeltSS,.id=c("immunity"))
  SP.R.SS=SP.R.SS[SP.R.SS$Var1 %in% c(0.4,0.7,1),]
  plotSusc=plot_ly(data=SP.R.SS,y=~Var1,x=~Var2/52,z=~value,type='scatter3d',
                   mode='lines',color=~Var1,
                   colors=c("chartreuse4","red","blue"),
                   linetype=~immunity,line=list(width=6))%>% 
    layout(title=titles[2],scene=scene,showlegend=F)
  if (id){
    times=seq(from=1,to=5*52,by=1)
    R0.list.b=rep(R0.list.a,length=length(times))
    R0.list1=R0.list.b/5
    R0.list1[times>tred1 & times<tred2]=0.6*R0.list1[times>tred1 & times<tred2]
    R0.list1[times>tred3 & times<tred4]=0.6*R0.list1[times>tred3 & times<tred4]
    primaryFOI=valuesSP[[7]]*R0.list1
    secondaryFOI=epsilonVector[7]*valuesSS[[7]]*R0.list1
    primaryInfection=valuesIP[[7]]
    secondaryInfection=valuesIS[[7]]
    
    primaryFOI_2=valuesSP[[2]]*R0.list1
    secondaryFOI_2=epsilonVector[2]*valuesSS[[2]]*R0.list1
    primaryInfection_2=valuesIP[[2]]
    secondaryInfection_2=valuesIS[[2]]
    
    typeValue=c(rep("$\\epsilon=1$",length(primaryFOI)),rep("$\\epsilon=0.5$",length(primaryFOI_2)))
    typeValue2=c(rep(" $\\epsilon=1$ ",length(primaryFOI)),rep(" $\\epsilon=0.5$ ",length(primaryFOI_2)))
    primary=c(primaryInfection,primaryInfection_2)
    secondary=c(secondaryInfection,secondaryInfection_2)
    FOI=c(primaryFOI,primaryFOI_2)
    FOI2=c(secondaryFOI,secondaryFOI_2)
    typeAll=c(typeValue,typeValue,typeValue2,typeValue2)
    xvalues=c(times,times,times,times,times,times,times,times)
    panelV=c(rep("$I_P$",length(typeValue)),rep("$I_S$",length(typeValue)),
             rep("$\\beta S_P$",length(typeValue)),rep("$\\epsilon \\beta S_S$",length(typeValue)))
    yvalues=c(primary,secondary,FOI,FOI2)
    dataF=data.frame(Years=xvalues/52,yy=yvalues,epsilon=typeAll,pp=panelV)
    f1=ggplot()+geom_line(data=dataF,aes(x=Years,y=yy,color=pp),size=1)+
      facet_wrap(~epsilon,ncol=2,scales='free')+theme_bw()+
      ylab('')+theme(legend.title=element_blank())#ylab("Fraction of population")++theme(legend.position="none")
    tikz(file="plot1_may15.tex",standAlone=T,width=5,height=3)
    print(f1)
    dev.off()
    
    #Adding burden on healthcare
    burden1=0.14*primaryInfection
    burden2a=0*secondaryInfection
    burden2b=0.07*secondaryInfection
    burden2c=0.14*secondaryInfection
    burden2d=0.21*secondaryInfection
    
    yy1=burden1+burden2a
    yy2=burden1+burden2b
    yy3=burden1+burden2c
    yy4=burden1+burden2d
    
    burden12=0.14*primaryInfection_2
    burden2a2=0*secondaryInfection_2
    burden2b2=0.07*secondaryInfection_2
    burden2c2=0.14*secondaryInfection_2
    burden2d2=0.21*secondaryInfection_2
    
    yy12=burden12+burden2a2
    yy22=burden12+burden2b2
    yy32=burden12+burden2c2
    yy42=burden12+burden2d2
    
    xvalues=c(times,times,times,times,times,times,times,times)
    yvalues=c(yy1,yy2,yy3,yy4,yy12,yy22,yy32,yy42)
    typeValue=c(rep("$\\epsilon=1$",length(xvalues)/2),rep("$\\epsilon=0.5$",length(xvalues)/2))
    secondaryBurden=c(rep("$0\\%$",length(times)),rep("$7\\%$",length(times)),rep("$14\\%$",length(times)),rep("$21\\%$",length(times)))
    
    secondaryBurden=factor(secondaryBurden,levels=c("$0\\%$","$7\\%$","$14\\%$","$21\\%$"))
    
    dataPanelE=data.frame(Years=xvalues/52,yy=yvalues,epsilon=typeValue,severe=secondaryBurden)
    
    f2=ggplot()+geom_line(data=dataPanelE,aes(x=Years,y=yy,color=severe,linetype=severe),size=1)+
      facet_wrap(~epsilon,ncol=1)+theme_bw()+
      scale_linetype_manual(values=c("solid","dashed","dotdash","twodash"))+
      ylab("Fraction of population\nwith severe disease")+
      labs(color="Percent\nof severe\nsecondary\ninfections",linetype="Percent\nof severe\nsecondary\ninfections")+
      theme(legend.title=element_text(size=12))
    tikz(file="plot_final.tex",standAlone=T,width=5,height=3)
    print(f2)
    dev.off()
  }
  plots=list(plotCombined,plotSusc)
  return(plots)
}

titles9=c("seasonality, soc dis long, infections","seasonality, soc dis long, immunity")
scenario1=plots.soc.dis.seasonality(1,15,68,0,0,FALSE,titles9)
scenario1[[1]]
scenario1[[2]]

titles10=c("seasonality, soc dis both, infections","seasonality, soc dis both, immunity")
scenario2=plots.soc.dis.seasonality(1,15,56,81,94,TRUE,titles10)
scenario2[[1]]
scenario2[[2]]

titles11=c("seasonality, soc dis short, infections","seasonality, soc dis short, immunity")
scenario3=plots.soc.dis.seasonality(1,15,56,0,0,FALSE,titles11)
scenario3[[1]]
scenario3[[2]]


###################SIR(S) model with seasonality, social distancing, and vaccination##################

sir_s_.model.seasonality.socdis.vax<-function (t,x,parameters) {
  
  SP=x[1]
  IP=x[2]
  R=x[3]
  SS=x[4]
  IS=x[5]
  V=x[6]
  
  N=parameters[["N"]]
  mu=parameters[["mu"]]
  delta=parameters[["delta"]]
  gamma=parameters[["gamma"]]
  R0.list=parameters[["R0.list"]]
  alpha=parameters[["alpha"]]
  epsilon=parameters[["epsilon"]]
  beta=R0.list[t]*gamma
  #Social distancing
  R0red=parameters[["R0red"]]
  tred1=parameters[["tred1"]]
  tred2=parameters[["tred2"]]
  #Vaccination
  nu=parameters[["nu"]]
  delta_vax=parameters[["delta_vax"]]
  tvax=parameters[["tvax"]]
  
  if((t > tred1) & (t < tred2)){
    beta = beta * R0red
  }
  
  svax=0
  if(t>tvax){
    svax=1
  }
  
  #Equations
  dSPdt=mu*N-(beta*SP*(IP+alpha*IS)/N)-mu*SP-svax*nu*SP
  dIPdt=(beta*SP*(IP+alpha*IS)/N)-(gamma+mu)*IP
  dRdt=gamma*(IP+IS)-(delta+mu)*R
  dSSdt=delta*R-epsilon*(beta*SS*(IP+alpha*IS)/N)-mu*SS+delta_vax*V-svax*nu*SS
  dISdt=epsilon*(beta*SS*(IP+alpha*IS)/N)-(gamma+mu)*IS
  dVdt=svax*nu*(SP+SS)-delta_vax*V-mu*V
  
  dxdt <- c(dSPdt,dIPdt,dRdt,dSSdt,dISdt,dVdt)
  list(dxdt)
}


plots.soc.dis.seasonality.vax=function(alpha,tred1,tred2,tred3,tred4,delta_vax,tvax,titles){
  nuVector=c(1e-3,5e-3,9e-3)
  valuesIP=list()
  valuesIS=list()
  valuesR=list()
  valuesSP=list()
  valuesSS=list()
  valuesV=list()
  delta=1/(1*52)
  N=1
  mu=1/(50*52)
  gamma=7/5
  R0red=0.6
  epsilon=0.7
  times=seq(from=1,to=5*52,by=1)
  R0.listOrig=read.table("NY.csv",header=T,sep=",")
  R0.list.a=R0.listOrig$hku1
  R0.list=rep(R0.list.a,length=length(times))
  I0=1e-9
  xstart=c(SP=1-I0,IP=I0,R=0,SS=0,IS=0,V=0) 
  for  (i in  1:length(nuVector)){
    nu=nuVector[i]
    parameters=list(N=N,mu=mu,gamma=gamma,alpha=alpha,
                    epsilon=epsilon,delta=delta,
                    R0.list=R0.list,R0red=R0red,tred1=tred1,
                    tred2=tred2,tred3=tred3,tred4=tred4,tvax=tvax,nu=nu,
                    delta_vax=delta_vax)
    out=as.data.frame(rk(func=sir_s_.model.seasonality.socdis.vax,y=xstart,times=times,parms=parameters))
    valuesIP[[i]]=out$IP
    valuesIS[[i]]=out$IS
    valuesR[[i]]=out$R
    valuesSS[[i]]=out$SS
    valuesSP[[i]]=out$SP
    valuesV[[i]]=out$V
  }
  dataValuesIP=t(do.call(cbind,valuesIP))
  dataValuesIS=t(do.call(cbind,valuesIS))
  dataValuesSS=t(do.call(cbind,valuesSS))
  dataValuesSP=t(do.call(cbind,valuesSP))
  dataValuesR=t(do.call(cbind,valuesR))
  #dataValuesV=t(do.call(cbind,valuesV))
  valuesMeltIP=melt(dataValuesIP)
  valuesMeltIS=melt(dataValuesIS)
  valuesMeltSS=melt(dataValuesSS)
  valuesMeltSP=melt(dataValuesSP)
  valuesMeltR=melt(dataValuesR)
  #valuesMeltV=melt(dataValuesV)
  for (i in 1:length(nuVector)){
    valuesMeltIP$Var1[valuesMeltIP$Var1==i]=nuVector[i]
    valuesMeltIS$Var1[valuesMeltIS$Var1==i]=nuVector[i]
    valuesMeltSS$Var1[valuesMeltSS$Var1==i]=nuVector[i]
    valuesMeltSP$Var1[valuesMeltSP$Var1==i]=nuVector[i]
    valuesMeltR$Var1[valuesMeltR$Var1==i]=nuVector[i]
    #valuesMeltV$Var1[valuesMeltV$Var1==i]=nuVector[i]
  }
  valuesMeltIP$Var1=as.character(valuesMeltIP$Var1)
  valuesMeltIS$Var1=as.character(valuesMeltIS$Var1)
  valuesMeltSS$Var1=as.character(valuesMeltSS$Var1)
  valuesMeltSP$Var1=as.character(valuesMeltSP$Var1)
  valuesMeltR$Var1=as.character(valuesMeltR$Var1)
  #valuesMeltV$Var1=as.character(valuesMeltV$Var1)
  
  scene=list(camera=list(eye=list(x=-1.5,y=-1.5,z=-1.5)),
             yaxis=list(autotick=FALSE,tickmode="array",tickvals=nuVector,title="ν",titlefont=list(size=25),tickfont=list(size=17),ticksuffix = "   ",showticklabels=FALSE,automargin=T),
             zaxis=list(title="Infections",titlefont=list(size=25),tickfont=list(size=17),ticksuffix = "   ",automargin=T),
             xaxis=list(title="Time (years)",titlefont=list(size=25),tickfont=list(size=17),ticksuffix = "   ",automargin=T))
  IP.IS=bind_rows(valuesMeltIP,valuesMeltIS,.id=c("infection"))
  IP.IS$infection[IP.IS$infection=="1"]="solid"
  IP.IS$infection[IP.IS$infection=="2"]="dashdot"
  plotCombined=plot_ly(data=IP.IS,y=~Var1,x=~Var2/52,z=~value,type='scatter3d',
                       mode='lines',color=~Var1,
                       colors=c("chartreuse4","red","blue"),
                       linetype=~I(infection),line=list(width=6))%>% 
    layout(title=titles[1],scene=scene,showlegend=F)
  
  scene=list(camera=list(eye=list(x=-1.5,y=-1.5,z=-1.5)),
             yaxis=list(autotick=FALSE,tickmode="array",tickvals=nuVector,title="ν",titlefont=list(size=25),tickfont=list(size=17),ticksuffix = "   ",showticklabels=FALSE,automargin=T),
             zaxis=list(title="Fraction of pop.",titlefont=list(size=25),tickfont=list(size=17),ticksuffix = "   ",automargin=T),
             xaxis=list(title="Time (years)",titlefont=list(size=25),tickfont=list(size=17),ticksuffix = "   ",automargin=T))
  #SP.R.SS.V=bind_rows(valuesMeltSP,valuesMeltR,valuesMeltSS,valuesMeltV,.id=c("immunity"))
  SP.R.SS=bind_rows(valuesMeltSP,valuesMeltR,valuesMeltSS,.id=c("immunity"))
  plotSusc=plot_ly(data=SP.R.SS,y=~Var1,x=~Var2/52,z=~value,type='scatter3d',
                   mode='lines',color=~Var1,
                   colors=c("chartreuse4","red","blue"),
                   linetype=~immunity,line=list(width=6))%>% 
    layout(title=titles[2],scene=scene,showlegend=F)
  
  forAreaPlotIP=valuesIP[[2]]
  forAreaPlotIS=valuesIS[[2]]
  forAreaPlotR=valuesR[[2]]
  forAreaPlotSS=valuesSS[[2]]
  forAreaPlotSP=valuesSP[[2]]
  forAreaPlotV=valuesV[[2]]
  
  data.AP.IP=data.frame(xx=times,yy=forAreaPlotIP)
  data.AP.IS=data.frame(xx=times,yy=forAreaPlotIS)
  data.AP.R=data.frame(xx=times,yy=forAreaPlotR)
  data.AP.SS=data.frame(xx=times,yy=forAreaPlotSS)
  data.AP.SP=data.frame(xx=times,yy=forAreaPlotSP)
  data.AP.V=data.frame(xx=times,yy=forAreaPlotV)
  
  data.AP.list=list("primary infection"=data.AP.IP,"secondary infection"=data.AP.IS,
       "fully immune"=data.AP.R,"partially immune"=data.AP.SS,"fully susceptible"=data.AP.SP,
       "vaccinated"=data.AP.V)
  
  area.plots.timecourses=bind_rows(data.AP.list,.id=c("immunity"))
  area.plots.timecourses$immunity=factor(area.plots.timecourses$immunity,levels=c("vaccinated","fully susceptible","fully immune",
                             "partially immune","secondary infection","primary infection"))
  sp.col=rgb(0.75,0.75,0.75)
  ip.col=rgb(0.8,0.28,0.23)
  r.col=rgb(0.36,0.33,0.58)
  ss.col=rgb(0.71,0.64,0.83)
  is.col=rgb(1,0.81,0.85)
  v.col=rgb(0.33,0.51,0.21)
  
  SP=area.plots.timecourses$`fully susceptible`
  plot.area.middle=ggplot(area.plots.timecourses,aes(x=xx/52,y=yy,fill=immunity))+geom_area()+
    scale_y_continuous(name="Fraction of population",expand=c(0,0))+
    scale_x_continuous(name="Time (years)",expand=c(0,0))+
    scale_fill_manual(name="",values=c(v.col,sp.col,r.col,ss.col,is.col,ip.col))
  plots=list(plotCombined,plotSusc,plot.area.middle)
  
  return(plots)
}

titles12=c("vaccination, half year, infections","vaccination, half year, immunity")
panels_de=plots.soc.dis.seasonality.vax(1,15,56,0,0,1/(0.5*52),78,titles12)
panels_de[[1]]
panels_de[[2]]

titles13=c("vaccination, one year, infections","vaccination, one year, immunity")
panels_fg=plots.soc.dis.seasonality.vax(1,15,56,0,0,1/(1*52),78,titles13)
panels_fg[[1]]
panels_fg[[2]]

titles14=c("vaccination, two years, infections","vaccination, two years, immunity")
panels_hi=plots.soc.dis.seasonality.vax(1,15,56,0,0,1/(2*52),78,titles14)
panels_hi[[1]]
panels_hi[[2]]


plots.soc.dis.seasonality.vax.together=function(alpha,tred1,tred2,tred3,
                                                tred4,delta,delta_vax,tvax,nuVector,
                                                primary.burden,secondary.burden,
                                                epsilon){
  valuesIP=list()
  valuesIS=list()
  valuesR=list()
  valuesSP=list()
  valuesSS=list()
  valuesV=list()
  plots=list()
  N=1
  mu=1/(50*52)
  gamma=7/5
  R0red=0.6
  times=seq(from=1,to=5*52,by=1)
  R0.listOrig=read.table("NY.csv",header=T,sep=",")
  R0.list.a=R0.listOrig$hku1
  R0.list=rep(R0.list.a,length=length(times))
  I0=1e-9
  sp.col=rgb(0.75,0.75,0.75)
  ip.col=rgb(0.8,0.28,0.23)
  r.col=rgb(0.36,0.33,0.58)
  ss.col=rgb(0.71,0.64,0.83)
  is.col=rgb(1,0.81,0.85)
  v.col=rgb(0.33,0.51,0.21)
  xstart=c(SP=1-I0,IP=I0,R=0,SS=0,IS=0,V=0) 
  for  (i in  1:length(nuVector)){
    nu=nuVector[i]
    parameters=list(N=N,mu=mu,gamma=gamma,alpha=alpha,
                    epsilon=epsilon,delta=delta,
                    R0.list=R0.list,R0red=R0red,tred1=tred1,
                    tred2=tred2,tred3=tred3,tred4=tred4,tvax=tvax,nu=nu,
                    delta_vax=delta_vax)
    out=as.data.frame(rk(func=sir_s_.model.seasonality.socdis.vax,
                         y=xstart,times=times,parms=parameters))
    valuesIP[[i]]=out$IP
    valuesIS[[i]]=out$IS
    valuesR[[i]]=out$R
    valuesSS[[i]]=out$SS
    valuesSP[[i]]=out$SP
    valuesV[[i]]=out$V
    forAreaPlotIP=valuesIP[[i]]
    forAreaPlotIS=valuesIS[[i]]
    forAreaPlotR=valuesR[[i]]
    forAreaPlotSS=valuesSS[[i]]
    forAreaPlotSP=valuesSP[[i]]
    forAreaPlotV=valuesV[[i]]
    
    burden=primary.burden*forAreaPlotIP+secondary.burden*forAreaPlotIS
    data.burden=data.frame(xx=times,yy=burden)
    plot.top=ggplot(data=data.burden)+geom_line(aes(x=xx/52,y=yy))+
      ylab("Severe cases")+xlab("Time (years)")+scale_x_continuous(expand=c(0,0))+
      theme_classic((size=14))
    
    data.AP.IP=data.frame(xx=times,yy=forAreaPlotIP)
    data.AP.IS=data.frame(xx=times,yy=forAreaPlotIS)
    data.AP.R=data.frame(xx=times,yy=forAreaPlotR)
    data.AP.SS=data.frame(xx=times,yy=forAreaPlotSS)
    data.AP.SP=data.frame(xx=times,yy=forAreaPlotSP)
    data.AP.V=data.frame(xx=times,yy=forAreaPlotV)
    
    data.AP.list=list("primary infection"=data.AP.IP,"secondary infection"=data.AP.IS,
                      "fully immune"=data.AP.R,"partially immune"=data.AP.SS,
                      "fully susceptible"=data.AP.SP,
                      "vaccinated"=data.AP.V)
    
    area.plots.timecourses=bind_rows(data.AP.list,.id=c("immunity"))
    area.plots.timecourses$immunity=factor(area.plots.timecourses$immunity,levels=c("vaccinated","fully susceptible","fully immune",
                                                                                 "partially immune","secondary infection","primary infection"))
    plot.below=ggplot(area.plots.timecourses,aes(x=xx/52,y=yy,fill=immunity))+geom_area()+
      theme_classic((size=14))+
      scale_y_continuous(name="Fraction of population",expand=c(0,0))+
      scale_x_continuous(name="Time (years)",expand=c(0,0))+
      scale_fill_manual(name="",values=c(v.col,sp.col,r.col,ss.col,is.col,ip.col))+theme(legend.position="none")
      combined=plot.top/plot.below+plot_layout(height=c(0.4,1))
      combined[[1]]=combined[[1]]+theme(axis.text.x=element_blank(),
                                      axis.title.x=element_blank())
      plots[[i]]=combined
  }
  return(plots)
}

plotsAP.scenario1=plots.soc.dis.seasonality.vax.together(1,15,56,0,0,1/(0.5*52),
                                                         1/(1*52),78,c(0),0.14,0.21,0.7)

plotsAP.scenario2=plots.soc.dis.seasonality.vax.together(1,15,56,0,0,1/(2*52),
                                                         1/(1*52),78,c(0),0.14,0.07,0.5)

plotsAP.scenario3=plots.soc.dis.seasonality.vax.together(1,15,56,0,0,1/(0.5*52),
                                                         1/(0.25*52),78,c(0.01),0.14,0.21,0.7)

plotsAP.scenario4=plots.soc.dis.seasonality.vax.together(1,15,56,0,0,1/(2*52),
                                                         1/(1*52),78,c(0.01),0.14,0.07,0.5)

AP.Plots=(((plotsAP.scenario1[[1]][[1]]/plotsAP.scenario1[[1]][[2]]+plot_layout(height=c(0.4,1)))|plot_spacer()|
            (plotsAP.scenario2[[1]][[1]]/plotsAP.scenario2[[1]][[2]]+plot_layout(height=c(0.4,1))))+plot_layout(widths=c(1,0.1,1)))/
  (((plotsAP.scenario3[[1]][[1]]/plotsAP.scenario3[[1]][[2]]+plot_layout(height=c(0.4,1)))|plot_spacer()|
     (plotsAP.scenario4[[1]][[1]]/plotsAP.scenario4[[1]][[2]])+plot_layout(height=c(0.4,1)))+plot_layout(widths=c(1,0.1,1)))
pdf("area_plots_june11_june22_final.pdf",width=10,height=12)
print(AP.Plots)
dev.off()





#####################################Pulse vaccination###################################


sir_s_.model.seasonality.socdis.vax.after.vaccination<-function (t,x,parameters) {
  
  SP=x[1]
  IP=x[2]
  R=x[3]
  SS=x[4]
  IS=x[5]
  V=x[6]
  
  N=parameters[["N"]]
  mu=parameters[["mu"]]
  delta=parameters[["delta"]]
  gamma=parameters[["gamma"]]
  R0.list=parameters[["R0.list"]]
  alpha=parameters[["alpha"]]
  epsilon=parameters[["epsilon"]]
  beta=R0.list[t]*gamma
  #Social distancing
  R0red=parameters[["R0red"]]
  tred1=parameters[["tred1"]]
  tred2=parameters[["tred2"]]
  delta_vax=parameters[["delta_vax"]]
  
  if((t > tred1) & (t < tred2)){
    beta = beta * R0red
  }
  
  #Equations
  dSPdt=mu*N-(beta*SP*(IP+alpha*IS)/N)-mu*SP
  dIPdt=(beta*SP*(IP+alpha*IS)/N)-(gamma+mu)*IP
  dRdt=gamma*(IP+IS)-(delta+mu)*R
  dSSdt=delta*R-epsilon*(beta*SS*(IP+alpha*IS)/N)-mu*SS+delta_vax*V
  dISdt=epsilon*(beta*SS*(IP+alpha*IS)/N)-(gamma+mu)*IS
  dVdt=-delta_vax*V-mu*V
  dxdt <- c(dSPdt,dIPdt,dRdt,dSSdt,dISdt,dVdt)
  list(dxdt)
}
plots.soc.dis.seasonality.vax.together.pulse=function(alpha,tred1,tred2,tred3,
                                                tred4,delta,delta_vax,tvax,nuVector,
                                                primary.burden,secondary.burden,
                                                epsilon){
  valuesIP=list()
  valuesIS=list()
  valuesR=list()
  valuesSP=list()
  valuesSS=list()
  valuesV=list()
  plots=list()
  N=1
  mu=1/(50*52)
  gamma=7/5
  R0red=0.6
  R0.listOrig=read.table("NY.csv",header=T,sep=",")
  R0.list.a=R0.listOrig$hku1
  R0.list=rep(R0.list.a,length=length(seq(from=1,to=5*52,by=1)))
  I0=1e-9
  sp.col=rgb(0.75,0.75,0.75)
  ip.col=rgb(0.8,0.28,0.23)
  r.col=rgb(0.36,0.33,0.58)
  ss.col=rgb(0.71,0.64,0.83)
  is.col=rgb(1,0.81,0.85)
  v.col=rgb(0.33,0.51,0.21)
  xstart=c(SP=1-I0,IP=I0,R=0,SS=0,IS=0,V=0) 
  times.init=seq(from=1,to=tvax,by=1)
  for  (i in  1:length(nuVector)){
    nu=nuVector[i]
    parameters=list(N=N,mu=mu,gamma=gamma,alpha=alpha,
                    epsilon=epsilon,delta=delta,
                    R0.list=R0.list,R0red=R0red,tred1=tred1,
                    tred2=tred2,tred3=tred3,tred4=tred4,tvax=tvax,nu=nu,
                    delta_vax=delta_vax)
    out.before=as.data.frame(rk(func=sir_s_.model.seasonality.socdis.vax,
                         y=xstart,times=times.init,parms=parameters))
    xstart.after.pulse=c(SP=(1-nu)*out.before$SP[tvax],IP=out.before$IP[tvax],R=out.before$R[tvax],
                         SS=(1-nu)*out.before$SS[tvax],IS=out.before$IS[tvax],
                         V=nu*(out.before$SP[tvax]+out.before$SS[tvax]))
    times.after=seq(from=tvax,to=5*52,by=1)
    out.after=as.data.frame(rk(func=sir_s_.model.seasonality.socdis.vax.after.vaccination,
                               y=xstart.after.pulse,times=times.after,parms=parameters))
    out=bind_rows(out.before[1:(tvax-1),],out.after)
    times=out$time
    valuesIP[[i]]=out$IP
    valuesIS[[i]]=out$IS
    valuesR[[i]]=out$R
    valuesSS[[i]]=out$SS
    valuesSP[[i]]=out$SP
    valuesV[[i]]=out$V
    forAreaPlotIP=valuesIP[[i]]
    forAreaPlotIS=valuesIS[[i]]
    forAreaPlotR=valuesR[[i]]
    forAreaPlotSS=valuesSS[[i]]
    forAreaPlotSP=valuesSP[[i]]
    forAreaPlotV=valuesV[[i]]
    
    burden=primary.burden*forAreaPlotIP+secondary.burden*forAreaPlotIS
    data.burden=data.frame(xx=times,yy=burden)
    plot.top=ggplot(data=data.burden)+geom_line(aes(x=xx/52,y=yy))+
      ylab("Severe cases")+xlab("Time (years)")+scale_x_continuous(expand=c(0,0))+
      theme_classic((size=14))
    
    data.AP.IP=data.frame(xx=times,yy=forAreaPlotIP)
    data.AP.IS=data.frame(xx=times,yy=forAreaPlotIS)
    data.AP.R=data.frame(xx=times,yy=forAreaPlotR)
    data.AP.SS=data.frame(xx=times,yy=forAreaPlotSS)
    data.AP.SP=data.frame(xx=times,yy=forAreaPlotSP)
    data.AP.V=data.frame(xx=times,yy=forAreaPlotV)
    
    data.AP.list=list("primary infection"=data.AP.IP,"secondary infection"=data.AP.IS,
                      "fully immune"=data.AP.R,"partially immune"=data.AP.SS,
                      "fully susceptible"=data.AP.SP,
                      "vaccinated"=data.AP.V)
    
    area.plots.timecourses=bind_rows(data.AP.list,.id=c("immunity"))
    area.plots.timecourses$immunity=factor(area.plots.timecourses$immunity,levels=c("vaccinated","fully susceptible","fully immune",
                                                                                    "partially immune","secondary infection","primary infection"))
    plot.below=ggplot(area.plots.timecourses,aes(x=xx/52,y=yy,fill=immunity))+geom_area()+
      theme_classic((size=14))+
      scale_y_continuous(name="Fraction of population",expand=c(0,0))+
      scale_x_continuous(name="Time (years)",expand=c(0,0))+
      scale_fill_manual(name="",values=c(v.col,sp.col,r.col,ss.col,is.col,ip.col))+theme(legend.position="none")
    combined=plot.top/plot.below+plot_layout(height=c(0.4,1))
    combined[[1]]=combined[[1]]+theme(axis.text.x=element_blank(),
                                      axis.title.x=element_blank())
    plots[[i]]=combined
  }
  return(plots)
}

plotsAP.scenario1.pulse=plots.soc.dis.seasonality.vax.together.pulse(1,15,56,0,0,1/(0.5*52),
                                                         1/(1*52),79,c(0),0.14,0.21,0.7)

plotsAP.scenario2.pulse=plots.soc.dis.seasonality.vax.together.pulse(1,15,56,0,0,1/(2*52),
                                                         1/(1*52),79,c(0),0.14,0.07,0.5)

plotsAP.scenario3.pulse=plots.soc.dis.seasonality.vax.together.pulse(1,15,56,0,0,1/(0.5*52),
                                                         1/(0.25*52),79,c(0.27),0.14,0.21,0.7)

plotsAP.scenario4.pulse=plots.soc.dis.seasonality.vax.together.pulse(1,15,56,0,0,1/(2*52),
                                                         1/(1*52),79,c(0.27),0.14,0.07,0.5)

AP.Plots.Vaccination=(((plotsAP.scenario1.pulse[[1]][[1]]/plotsAP.scenario1.pulse[[1]][[2]]+plot_layout(height=c(0.4,1)))|plot_spacer()|
             (plotsAP.scenario2.pulse[[1]][[1]]/plotsAP.scenario2.pulse[[1]][[2]]+plot_layout(height=c(0.4,1))))+plot_layout(widths=c(1,0.1,1)))/
  (((plotsAP.scenario3.pulse[[1]][[1]]/plotsAP.scenario3.pulse[[1]][[2]]+plot_layout(height=c(0.4,1)))|plot_spacer()|
      (plotsAP.scenario4.pulse[[1]][[1]]/plotsAP.scenario4.pulse[[1]][[2]])+plot_layout(height=c(0.4,1)))+plot_layout(widths=c(1,0.1,1)))
pdf("area_plots_pulse_final.pdf",width=10,height=12)
print(AP.Plots.Vaccination)
dev.off()


