#############################################Packages required########################################
library(deSolve)
library(ggplot2)
library(reshape2)
library(plotly)
library(dplyr)
library(tikzDevice)
library(pracma)
library(cowplot)
library(patchwork)



##########################SIR(S) model with seasonality and social distancing#########################
########Modified so that second social distancing part occurs when "greater or equal to tred3", instead of
########"greater than tred3" so that at exactly 26 weeks, the value fits within this interval.
sir_s_.model.seasonality.socdis.ramp=function(t,x,parameters){
  
  SP=x[1]
  IP=x[2]
  R=x[3]
  SS=x[4]
  IS=x[5]
  
  N=parameters[["N"]]
  mu=parameters[["mu"]]
  delta=parameters[["delta"]]
  gamma=parameters[["gamma"]]
  R0.list=parameters[["R0.list"]]
  alpha=parameters[["alpha"]]
  epsilon=parameters[["epsilon"]]
  beta=R0.list[t]*gamma
  #Social distancing
  R0red=parameters[["R0red"]]
  R0redExtra=parameters[["R0redExtra"]]
  tred1=parameters[["tred1"]]
  tred2=parameters[["tred2"]]
  tred3=parameters[["tred3"]]
  tred4=parameters[["tred4"]]
  
  if((t>tred1) & (t<tred2)){
    beta = beta * R0red
  }
  if((t>=tred3) & (t<tred4)){
    beta = beta * R0redExtra
  }
  
  dSPdt=mu*N-(beta*SP*(IP+alpha*IS)/N)-mu*SP
  dIPdt=(beta*SP*(IP+alpha*IS)/N)-(gamma+mu)*IP
  dRdt=gamma*(IP+IS)-(delta+mu)*R
  dSSdt=delta*R-epsilon*(beta*SS*(IP+alpha*IS)/N)-mu*SS
  dISdt=epsilon*(beta*SS*(IP+alpha*IS)/N)-(gamma+mu)*IS
  
  dxdt <- c(dSPdt,dIPdt,dRdt,dSSdt,dISdt)
  list(dxdt)
}


#####################Plots for SIR(S)  model with seasonality and social distancing###################
#####The id variable is to distinguish when to plot the extra plots, panels D and E
#####of Fig. 3. If id=TRUE, latex code will be generated for these plots, which can
#####then be compiled to produce the panels.
plots.soc.dis.seasonality.ramp=function(alpha,tred1,tred2,tred3,tred4,id,titles,R0redExtra){
  epsilonVector=c(0.4,0.5,0.6,0.7,0.8,0.9,1)
  valuesIP=list()
  valuesIS=list()
  valuesR=list()
  valuesSP=list()
  valuesSS=list()
  delta=1/(1*52)
  N=1
  mu=1/(50*52)
  gamma=7/5
  R0red=0.6
  times=seq(from=1,to=5*52,by=1)
  R0.listOrig=read.table("NY.csv",header=T,sep=",")
  R0.list.a=R0.listOrig$hku1
  R0.list=rep(R0.list.a,length=length(times))
  I0=1e-9
  xstart=c(SP=1-I0,IP=I0,R=0,SS=0,IS=0) 
  for  (i in  1:length(epsilonVector)){
    epsilon=epsilonVector[i]
    parameters=list(N=N,mu=mu,gamma=gamma,alpha=alpha,
                    epsilon=epsilon,delta=delta,
                    R0.list=R0.list,R0red=R0red,tred1=tred1,
                    tred2=tred2,tred3=tred3,tred4=tred4,R0redExtra=R0redExtra)
    out=as.data.frame(rk(func=sir_s_.model.seasonality.socdis.ramp,y=xstart,times=times,parms=parameters))
    valuesIP[[i]]=out$IP
    valuesIS[[i]]=out$IS
    valuesR[[i]]=out$R
    valuesSS[[i]]=out$SS
    valuesSP[[i]]=out$SP
  }
  dataValuesIP=t(do.call(cbind,valuesIP))
  dataValuesIS=t(do.call(cbind,valuesIS))
  dataValuesSS=t(do.call(cbind,valuesSS))
  dataValuesSP=t(do.call(cbind,valuesSP))
  dataValuesR=t(do.call(cbind,valuesR))
  valuesMeltIP=melt(dataValuesIP)
  valuesMeltIS=melt(dataValuesIS)
  valuesMeltSS=melt(dataValuesSS)
  valuesMeltSP=melt(dataValuesSP)
  valuesMeltR=melt(dataValuesR)
  for (i in 1:length(epsilonVector)){
    valuesMeltIP$Var1[valuesMeltIP$Var1==i]=epsilonVector[i]
    valuesMeltIS$Var1[valuesMeltIS$Var1==i]=epsilonVector[i]
    valuesMeltSS$Var1[valuesMeltSS$Var1==i]=epsilonVector[i]
    valuesMeltSP$Var1[valuesMeltSP$Var1==i]=epsilonVector[i]
    valuesMeltR$Var1[valuesMeltR$Var1==i]=epsilonVector[i]
  }
  valuesMeltIP$Var1=as.character(valuesMeltIP$Var1)
  valuesMeltIS$Var1=as.character(valuesMeltIS$Var1)
  valuesMeltSS$Var1=as.character(valuesMeltSS$Var1)
  valuesMeltSP$Var1=as.character(valuesMeltSP$Var1)
  valuesMeltR$Var1=as.character(valuesMeltR$Var1)
  
  scene=list(camera=list(eye=list(x=-1.5,y=-1.5,z=-1.5)),
             yaxis=list(title="ϵ",titlefont=list(size=25),tickfont=list(size=17),ticksuffix = "   ",automargin=T),
             zaxis=list(title="Infections",titlefont=list(size=25),tickfont=list(size=17),ticksuffix = "   ",automargin=T),
             xaxis=list(title="Time (years)",titlefont=list(size=25),tickfont=list(size=17),ticksuffix = "   ",automargin=T))
  IP.IS=bind_rows(valuesMeltIP,valuesMeltIS,.id=c("infection"))
  IP.IS$infection[IP.IS$infection=="1"]="solid"
  IP.IS$infection[IP.IS$infection=="2"]="dashdot"
  plotCombined=plot_ly(data=IP.IS,y=~Var1,x=~Var2/52,z=~value,type='scatter3d',
                       mode='lines',color=~Var1,
                       colors=c("chartreuse4","darkgoldenrod1","tan2","red",'cornsilk4',"darkorchid1","blue"),
                       linetype=~I(infection),line=list(width=6))%>% 
    layout(title=titles[1],scene=scene,showlegend=F)
  
  scene=list(camera=list(eye=list(x=-1.5,y=-1.5,z=-1.5)),
             yaxis=list(title="ϵ",titlefont=list(size=25),tickfont=list(size=17),ticksuffix = "   ",automargin=T),
             zaxis=list(title="Fraction of pop.",titlefont=list(size=25),tickfont=list(size=17),ticksuffix = "   ",automargin=T),
             xaxis=list(title="Time (years)",titlefont=list(size=25),tickfont=list(size=17),ticksuffix = "   ",automargin=T))
  SP.R.SS=bind_rows(valuesMeltSP,valuesMeltR,valuesMeltSS,.id=c("immunity"))
  SP.R.SS=SP.R.SS[SP.R.SS$Var1 %in% c(0.4,0.7,1),]
  plotSusc=plot_ly(data=SP.R.SS,y=~Var1,x=~Var2/52,z=~value,type='scatter3d',
                   mode='lines',color=~Var1,
                   colors=c("chartreuse4","red","blue"),
                   linetype=~immunity,line=list(width=6))%>% 
    layout(title=titles[2],scene=scene,showlegend=F)
  if (id){
    times=seq(from=1,to=5*52,by=1)
    R0.list.b=rep(R0.list.a,length=length(times))
    R0.list1=R0.list.b/5
    R0.list1[times>tred1 & times<tred2]=0.6*R0.list1[times>tred1 & times<tred2]
    primaryFOI=valuesSP[[7]]*R0.list1
    secondaryFOI=epsilonVector[7]*valuesSS[[7]]*R0.list1
    primaryInfection=valuesIP[[7]]
    secondaryInfection=valuesIS[[7]]
    
    primaryFOI_2=valuesSP[[2]]*R0.list1
    secondaryFOI_2=epsilonVector[2]*valuesSS[[2]]*R0.list1
    primaryInfection_2=valuesIP[[2]]
    secondaryInfection_2=valuesIS[[2]]
    
    typeValue=c(rep("$\\epsilon=1$",length(primaryFOI)),rep("$\\epsilon=0.5$",length(primaryFOI_2)))
    typeValue2=c(rep(" $\\epsilon=1$ ",length(primaryFOI)),rep(" $\\epsilon=0.5$ ",length(primaryFOI_2)))
    primary=c(primaryInfection,primaryInfection_2)
    secondary=c(secondaryInfection,secondaryInfection_2)
    FOI=c(primaryFOI,primaryFOI_2)
    FOI2=c(secondaryFOI,secondaryFOI_2)
    typeAll=c(typeValue,typeValue,typeValue2,typeValue2)
    xvalues=c(times,times,times,times,times,times,times,times)
    panelV=c(rep("$I_P$",length(typeValue)),rep("$I_S$",length(typeValue)),
             rep("$\\beta S_P$",length(typeValue)),rep("$\\epsilon \\beta S_S$",length(typeValue)))
    yvalues=c(primary,secondary,FOI,FOI2)
    dataF=data.frame(Years=xvalues/52,yy=yvalues,epsilon=typeAll,pp=panelV)
    f1=ggplot()+geom_line(data=dataF,aes(x=Years,y=yy,color=pp),size=1)+
      facet_wrap(~epsilon,ncol=2,scales='free')+theme_bw()+
      ylab('')+theme(legend.title=element_blank())#ylab("Fraction of population")++theme(legend.position="none")
    tikz(file="plot1_may15.tex",standAlone=T,width=5,height=3)
    print(f1)
    dev.off()
    
    #Adding burden on healthcare
    burden1=0.14*primaryInfection
    burden2a=0*secondaryInfection
    burden2b=0.07*secondaryInfection
    burden2c=0.14*secondaryInfection
    burden2d=0.21*secondaryInfection
    
    yy1=burden1+burden2a
    yy2=burden1+burden2b
    yy3=burden1+burden2c
    yy4=burden1+burden2d
    
    burden12=0.14*primaryInfection_2
    burden2a2=0*secondaryInfection_2
    burden2b2=0.07*secondaryInfection_2
    burden2c2=0.14*secondaryInfection_2
    burden2d2=0.21*secondaryInfection_2
    
    yy12=burden12+burden2a2
    yy22=burden12+burden2b2
    yy32=burden12+burden2c2
    yy42=burden12+burden2d2
    
    xvalues=c(times,times,times,times,times,times,times,times)
    yvalues=c(yy1,yy2,yy3,yy4,yy12,yy22,yy32,yy42)
    typeValue=c(rep("$\\epsilon=1$",length(xvalues)/2),rep("$\\epsilon=0.5$",length(xvalues)/2))
    secondaryBurden=c(rep("$0\\%$",length(times)),rep("$7\\%$",length(times)),rep("$14\\%$",length(times)),rep("$21\\%$",length(times)))
    
    secondaryBurden=factor(secondaryBurden,levels=c("$0\\%$","$7\\%$","$14\\%$","$21\\%$"))
    
    dataPanelE=data.frame(Years=xvalues/52,yy=yvalues,epsilon=typeValue,severe=secondaryBurden)
    
    f2=ggplot()+geom_line(data=dataPanelE,aes(x=Years,y=yy,color=severe,linetype=severe),size=1)+
      facet_wrap(~epsilon,ncol=1)+theme_bw()+
      scale_linetype_manual(values=c("solid","dashed","dotdash","twodash"))+
      ylab("Fraction of population\nwith severe disease")+
      labs(color="Percent\nof severe\nsecondary\ninfections",linetype="Percent\nof severe\nsecondary\ninfections")+
      theme(legend.title=element_text(size=12))
    tikz(file="plot2_may15.tex",standAlone=T,width=5,height=3)
    print(f2)
    dev.off()
  }
  plots=list(plotCombined,plotSusc)
  return(plots)
}

titles.ramp1=c("seasonality, soc dis ramp 70%, infections","seasonality, soc dis ramp 70%, immunity")
scenario.ramp1=plots.soc.dis.seasonality.ramp(1,15,26,26,56,FALSE,titles.ramp1,0.7)
scenario.ramp1[[1]]

titles.ramp2=c("seasonality, soc dis ramp 80%, infections","seasonality, soc dis ramp 80%, immunity")
scenario.ramp2=plots.soc.dis.seasonality.ramp(1,15,26,26,56,FALSE,titles.ramp2,0.8)
scenario.ramp2[[1]]

titles.ramp3=c("seasonality, soc dis ramp 90%, infections","seasonality, soc dis ramp 90%, immunity")
scenario.ramp3=plots.soc.dis.seasonality.ramp(1,15,26,26,56,FALSE,titles.ramp3,0.9)
scenario.ramp3[[1]]

titles.ramp4=c("seasonality, soc dis ramp 100%, infections","seasonality, soc dis ramp 100%, immunity")
scenario.ramp4=plots.soc.dis.seasonality.ramp(1,15,26,26,56,FALSE,titles.ramp4,1)
scenario.ramp4[[1]]
