library(deSolve)
library(ggplot2)
library(reshape2)
library(pracma)
library(grid)
library(gridExtra)
library(plotly)
library(gplots)
library(foreach)
library(doParallel)
library(abind)
library(RColorBrewer)

##########################################################################################
######################################Code for Plotting###################################
##########################################################################################
filled.contour3 <- function (x = seq(0, 1, length.out = nrow(z)),
                             y = seq(0, 1, length.out = ncol(z)), z, xlim = range(x, finite = TRUE), 
                             ylim = range(y, finite = TRUE), zlim = range(z, finite = TRUE), 
                             levels = pretty(zlim, nlevels), nlevels = 20, color.palette = cm.colors, 
                             col = color.palette(length(levels) - 1), plot.title, plot.axes, 
                             key.title, key.axes, asp = NA, xaxs = "i", yaxs = "i", las = 1, 
                             axes = TRUE, frame.plot = axes,mar, ...) 
{
  # modification by Ian Taylor of the filled.contour function
  # to remove the key and facilitate overplotting with contour()
  # further modified by Carey McGilliard and Bridget Ferris
  # to allow multiple plots on one page
  
  if (missing(z)) {
    if (!missing(x)) {
      if (is.list(x)) {
        z <- x$z
        y <- x$y
        x <- x$x
      }
      else {
        z <- x
        x <- seq.int(0, 1, length.out = nrow(z))
      }
    }
    else stop("no 'z' matrix specified")
  }
  else if (is.list(x)) {
    y <- x$y
    x <- x$x
  }
  if (any(diff(x) <= 0) || any(diff(y) <= 0)) 
    stop("increasing 'x' and 'y' values expected")
  # mar.orig <- (par.orig <- par(c("mar", "las", "mfrow")))$mar
  # on.exit(par(par.orig))
  # w <- (3 + mar.orig[2]) * par("csi") * 2.54
  # par(las = las)
  # mar <- mar.orig
  plot.new()
  # par(mar=mar)
  plot.window(xlim, ylim, "", xaxs = xaxs, yaxs = yaxs, asp = asp)
  if (!is.matrix(z) || nrow(z) <= 1 || ncol(z) <= 1) 
    stop("no proper 'z' matrix specified")
  if (!is.double(z)) 
    storage.mode(z) <- "double"
  .filled.contour(as.double(x), as.double(y), z, as.double(levels), 
                          col = col)
  if (missing(plot.axes)) {
    if (axes) {
      title(main = "", xlab = "", ylab = "")
      Axis(x, side = 1)
      Axis(y, side = 2)
    }
  }
  else plot.axes
  if (frame.plot) 
    box()
  if (missing(plot.title)) 
    title(...)
  else plot.title
  invisible()
}

filled.legend <- function (x = seq(0, 1, length.out = nrow(z)), y = seq(0, 1, length.out = ncol(z)), z, 
                           xlim = range(x, finite = TRUE), 
                           ylim = range(y, finite = TRUE), zlim = range(z, finite = TRUE), 
                           levels = pretty(zlim, nlevels), nlevels = 20, color.palette = cm.colors, 
                           col = color.palette(length(levels) - 1), plot.title, plot.axes, 
                           key.title, key.axes, asp = NA, xaxs = "i", yaxs = "i", las = 1, 
                           axes = TRUE, frame.plot = axes, ...) 
{
  # modification of filled.contour by Carey McGilliard and Bridget Ferris
  # designed to just plot the legend
  if (missing(z)) {
    if (!missing(x)) {
      if (is.list(x)) {
        z <- x$z
        y <- x$y
        x <- x$x
      }
      else {
        z <- x
        x <- seq.int(0, 1, length.out = nrow(z))
      }
    }
    else stop("no 'z' matrix specified")
  }
  else if (is.list(x)) {
    y <- x$y
    x <- x$x
  }
  if (any(diff(x) <= 0) || any(diff(y) <= 0)) 
    stop("increasing 'x' and 'y' values expected")
  #  mar.orig <- (par.orig <- par(c("mar", "las", "mfrow")))$mar
  #  on.exit(par(par.orig))
  #  w <- (3 + mar.orig[2L]) * par("csi") * 2.54
  #layout(matrix(c(2, 1), ncol = 2L), widths = c(1, lcm(w)))
  #  par(las = las)
  #  mar <- mar.orig
  #  mar[4L] <- mar[2L]
  #  mar[2L] <- 1
  #  par(mar = mar)
  # plot.new()
  plot.window(xlim = c(0, 1), ylim = range(levels), xaxs = "i", 
              yaxs = "i")
  rect(0, levels[-length(levels)], 1, levels[-1L], col = col)
  if (missing(key.axes)) {
    if (axes) 
      axis(4)
  }
  else key.axes
  box()
}

##########################################################################################
######################################ODE MODELS##########################################
##########################################################################################

####################
##No seasonality
####################
sir_s_.model.socdis<-function (t,x,parameters) {
  SP=x[1]
  IP=x[2]
  R=x[3]
  SS=x[4]
  IS=x[5]
  
  #Parameters
  N=parameters["N"]
  mu=parameters["mu"]
  delta=parameters["delta"]
  gamma=parameters["gamma"]
  beta=parameters["beta"]
  alpha=parameters["alpha"]
  epsilon=parameters["epsilon"]
  R0red=parameters["R0red"]
  tred1=parameters["tred1"]
  tred2=parameters["tred2"]
  tred3=parameters["tred3"]
  tred4=parameters["tred4"]
  
  if(((t > tred1) & (t < tred2))|((t > tred3) & (t < tred4))){
    beta = beta * R0red
  }
  
  #Equations
  dSPdt=mu*N-(beta*SP*(IP+alpha*IS)/N)-mu*SP
  dIPdt=(beta*SP*(IP+alpha*IS)/N)-(gamma+mu)*IP
  dRdt=gamma*(IP+IS)-(delta+mu)*R
  dSSdt=delta*R-epsilon*(beta*SS*(IP+alpha*IS)/N)-mu*SS
  dISdt=epsilon*(beta*SS*(IP+alpha*IS)/N)-(gamma+mu)*IS
  dxdt <- c(dSPdt,dIPdt,dRdt,dSSdt,dISdt)
  list(dxdt)
}

####################
##Seasonality
####################
sir_s_.model.seasonality.socdis<-function (t,x,parameters) {
 
  SP=x[1]
  IP=x[2]
  R=x[3]
  SS=x[4]
  IS=x[5]
  
  N=parameters[["N"]]
  mu=parameters[["mu"]]
  delta=parameters[["delta"]]
  gamma=parameters[["gamma"]]
  R0.list=parameters[["R0.list"]]
  alpha=parameters[["alpha"]]
  epsilon=parameters[["epsilon"]]
  beta=R0.list[t]*gamma
  #Social distancing
  R0red=parameters[["R0red"]]
  tred1=parameters[["tred1"]]
  tred2=parameters[["tred2"]]
  tred3=parameters[["tred3"]]
  tred4=parameters[["tred4"]]
  
  if(((t > tred1) & (t < tred2))|((t > tred3) & (t < tred4))){
    beta = beta * R0red
  }
  
  dSPdt=mu*N-(beta*SP*(IP+alpha*IS)/N)-mu*SP
  dIPdt=(beta*SP*(IP+alpha*IS)/N)-(gamma+mu)*IP
  dRdt=gamma*(IP+IS)-(delta+mu)*R
  dSSdt=delta*R-epsilon*(beta*SS*(IP+alpha*IS)/N)-mu*SS
  dISdt=epsilon*(beta*SS*(IP+alpha*IS)/N)-(gamma+mu)*IS
  
  dxdt <- c(dSPdt,dIPdt,dRdt,dSSdt,dISdt)
  list(dxdt)
}

##########################################################################################
######################################Data input##########################################
##########################################################################################
NYdata=read.table("NY.csv",header=T,sep=",")
hku1=NYdata$hku1

##########################################################################################
######################################PLOTS###############################################
##########################################################################################

############################################################
#############Time lag between first 2 peaks#################
############################################################


####################
##No seasonality
####################
#Parameters
timenum = 365
minpeakval = 0.01
gridsize = 50

N=1
mu=1/(50*timenum)
gamma=1/5
beta=mean(hku1/(1/gamma))
R0red=1
tred1=30
tred2=60
tred3=0
tred4=0

alpha=0.8
  
maxvalp2=function(epsilon,ImmuneTime){

  delta=1/ImmuneTime
  
  parameters<-c(N=N,mu=mu,gamma=gamma,beta=beta,alpha=alpha,
                epsilon=epsilon,delta=delta,R0red=R0red,tred1=tred1,tred2=tred2,
                tred3=tred3,tred4=tred4)
      
  times<-seq(from=1,to=timenum*5,by=1)
  I0=1e-9
  xstart<-c(SP=1-I0,IP=I0,R=0,SS=0,IS=0) 
    
  out<-as.data.frame(rk(xstart,times, sir_s_.model.socdis , parameters))
      
  Iseries <- out$IS + out$IP
  Ipeaks <- findpeaks(Iseries, minpeakheight = minpeakval)
  #print(Ipeaks)
  if(length(Ipeaks[,1])>1){
    temlen <- length(Ipeaks[,1])
    elim <- matrix(0, 1, length(Ipeaks[,1]))
    elimnum <- 0
    for(ind in 1:length(Ipeaks[,1])){
      if(abs(Ipeaks[ind,1]-Iseries[Ipeaks[ind,3]])<0.0001){
        elimnum <- elimnum + 1
        elim[elimnum] <- ind
      }
    }
    if(elimnum>0){
      elim <- elim[-seq((elimnum+1),length(Ipeaks[,1]),1)]
      if(temlen-elimnum > 1){
        Ipeaks <- Ipeaks[-elim,]
      }else{
        Ipeaks <- t(as.matrix(Ipeaks[-elim,]))
      }
    }
    
  }
  #plot(times,Iseries)
  #print(Ipeaks)
  if(length(Ipeaks[,1])>1){
    maxp2 <- Ipeaks[2,1]
  }else{
    maxp2 <- 10000
  }
  value=maxp2
  #disp(value)
  return(value)
}
    
Maxvalp2=Vectorize(maxvalp2)
  
epsilon=seq(from=0,to=1,length.out=gridsize)
ImmuneTime=seq(from=0.1*timenum,to=2*timenum,length.out=gridsize)
  
#epsilon=1
#ImmuneTime=2*timenum
    
output=outer(epsilon,ImmuneTime,Maxvalp2)
out_maxvalp2_nos <- output

##########Plot with filled contour
rgb.palette <- c(colorRampPalette(brewer.pal(10,"GnBu"))(10),"grey35")
plot.new()
pdf(sprintf('Maxvalp2_nos_alpha0p8.pdf'),height=4,width=6)
par(mar=c(5.1, 4.1, 4.1, 2.1), tck=-0.02, cex.axis = 1 ,cex.main = 1.5)
par(mfrow=c(2,3), mai = c(0.4, 0.3, 0.4, 0.3))
par(mgp=c(2, 1, 0))
for (i in 1:1){
  
  filled.contour3(x=epsilon,y=ImmuneTime/timenum,z=out_maxvalp2_nos,
                 levels = c(pretty(c(0,0.1), 8),10000), 
                 col = rgb.palette,
                 plot.title=title(main = "",
                                  xlab="", 
                                  ylab="")
  )
}
par(new = "FALSE",plt = c(0.9,1.1,0.2,0.8),las = 1,cex.axis = 1)
filled.legend(x=epsilon,y=ImmuneTime/timenum,z=matrix(0,length(epsilon),length(ImmuneTime)),
                levels = c(pretty(c(0,0.1), 8)), 
                col = rgb.palette,
                plot.title=title(main = "", xlab="", ylab=""))
dev.off()

####################
##Seasonality
####################
timenum = 52
minpeakval = 0.01
gridsize = 50

N=1
mu=1/(50*52)
gamma=7/5
#Social distancing
R0red=1
tred1=2
tred2=3
tred3=0
tred4=0
#Seasonality
times<-seq(from=1,to=5*timenum,by=1)
R0.list <- rep(hku1,length=length(times))
##If you want to shift seasonal cycle do it here
seasonlag <- 38
R0.list <- circshift(R0.list,-seasonlag)

alpha=1

maxvalp2=function(epsilon,ImmuneTime){
  
  delta=1/ImmuneTime

  parameters<-list(N=N,mu=mu,gamma=gamma,alpha=alpha,
                epsilon=epsilon,delta=delta,R0.list = R0.list,R0red=R0red,
                tred1=tred1,tred2=tred2,tred3=tred3,tred4=tred4)
  
  I0=1e-9
  xstart<-c(SP=1-I0,IP=I0,R=0,SS=0,IS=0) 
  times<-seq(from=1,to=5*timenum,by=1)
  
  out<-as.data.frame(rk(xstart,times, sir_s_.model.seasonality.socdis , parameters))
  
  Iseries <- out$IS + out$IP
  Ipeaks <- findpeaks(Iseries, minpeakheight = minpeakval)
  #print(Ipeaks)
  if(length(Ipeaks[,1])>1){
    temlen <- length(Ipeaks[,1])
    elim <- matrix(0, 1, length(Ipeaks[,1]))
    elimnum <- 0
    for(ind in 1:length(Ipeaks[,1])){
      if(abs(Ipeaks[ind,1]-Iseries[Ipeaks[ind,3]])<0.0001){
        elimnum <- elimnum + 1
        elim[elimnum] <- ind
      }
    }
    if(elimnum>0){
      elim <- elim[-seq((elimnum+1),length(Ipeaks[,1]),1)]
      if(temlen-elimnum > 1){
        Ipeaks <- Ipeaks[-elim,]
      }else{
        Ipeaks <- t(as.matrix(Ipeaks[-elim,]))
      }
    }
    
  }
  #plot(times,Iseries)
  #print(Ipeaks)
  if(length(Ipeaks[,1])>1){
    maxp2 <- Ipeaks[2,1]
  }else{
    maxp2 <- 10000
  }
  value=maxp2
  #disp(value)
  return(value)
}

Maxvalp2=Vectorize(maxvalp2)

epsilon=seq(from=0,to=1,length.out=gridsize)
ImmuneTime=seq(from=0.1*timenum,to=2*timenum,length.out=gridsize)

#epsilon=1
#ImmuneTime=0.1*timenum

output=outer(epsilon,ImmuneTime,Maxvalp2)
out_maxvalp2_s <- output

##########Plot with filled contour
rgb.palette <- c(colorRampPalette(brewer.pal(10,"GnBu"))(10),"grey35")

plot.new()
pdf(sprintf('Maxvalp2_s38_alpha1.pdf'),height=4,width=6)
par(mar=c(5.1, 4.1, 4.1, 2.1), tck=-0.02, cex.axis = 1 ,cex.main = 1.5)
par(mfrow=c(2,3), mai = c(0.4, 0.3, 0.4, 0.3))
par(mgp=c(2, 1, 0))
for (i in 1:1){

  filled.contour3(x=epsilon,y=ImmuneTime/timenum,z=out_maxvalp2_s,
                  levels = c(pretty(c(0,0.1), 8),10000),
                  col = rgb.palette,
                  plot.title=title(main = "",
                                   xlab="",
                                   ylab="")
  )
}
par(new = "FALSE",plt = c(0.9,1.1,0.2,0.8),las = 1,cex.axis = 1)
filled.legend(x=epsilon,y=ImmuneTime/timenum,z=matrix(0,length(epsilon),length(ImmuneTime)),
              levels = pretty(c(0,0.1), 8),
              col = rgb.palette,
              plot.title=title(main = "", xlab="", ylab=""))

dev.off()



#####Plot the R_0 values
#No seasonality
plot.new()
pdf(sprintf('R0_Noseasonality.pdf'),height=4,width=6)
par(mar=c(5.1, 4.1, 4.1, 2.1), tck=-0.04, cex.axis = 1.2, cex.lab=1.5)
par(mgp=c(2.5, 1, 0))
ylab.text = expression('R'['0'])
plot(seq(1,52,1),rep(mean(hku1),52),type="l",lwd=1.5, ylim=c(0.5,2.5), xlab="Week",ylab=ylab.text)
dev.off()
#Seasonality
plot.new()
pdf(sprintf('R0_Seasonality.pdf'),height=4,width=6)
par(mar=c(5.1, 4.1, 4.1, 2.1), tck=-0.04, cex.axis = 1.2, cex.lab=1.5)
par(mgp=c(2.5, 1, 0))
ylab.text = expression('R'['0'])
plot(seq(1,52,1),hku1,type="l",lwd=1.5, ylim=c(0.5,2.5), xlab="Week",ylab=ylab.text)
abline(v=c(13,26,39),col=c("black","black","black"),lty=c(2,2,2),lwd=c(1.5,1.5,1.5))
dev.off()
#Social distancing
#V1
xax_time <- seq(1,52*5,1)
yax_r0 <- rep(hku1,5)
yax_r0[16:67] <- 0.6 * yax_r0[16:67]
plot.new()
pdf(sprintf('R0_Seasonality_SocDist1.pdf'),height=4,width=6)
par(mar=c(5.1, 4.1, 4.1, 2.1), tck=-0.04, cex.axis = 1.2, cex.lab=1.5)
par(mgp=c(2.5, 1, 0))
ylab.text = expression('R'['0'])
plot(xax_time,yax_r0,type="l",lwd=1.5, ylim=c(0.5,2.5), xlab="Week",ylab=ylab.text)
dev.off()
#V2
xax_time <- seq(1,52*5,1)
yax_r0 <- rep(hku1,5)
yax_r0[16:55] <- 0.6 * yax_r0[16:55]
plot.new()
pdf(sprintf('R0_Seasonality_SocDist2.pdf'),height=4,width=6)
par(mar=c(5.1, 4.1, 4.1, 2.1), tck=-0.04, cex.axis = 1.2, cex.lab=1.5)
par(mgp=c(2.5, 1, 0))
ylab.text = expression('R'['0'])
plot(xax_time,yax_r0,type="l",lwd=1.5, ylim=c(0.5,2.5), xlab="Week",ylab=ylab.text)
dev.off()
#V3
xax_time <- seq(1,52*5,1)
yax_r0 <- rep(hku1,5)
yax_r0[16:55] <- 0.6 * yax_r0[16:55]
yax_r0[82:93] <- 0.6 * yax_r0[82:93]
plot.new()
pdf(sprintf('R0_Seasonality_SocDist3.pdf'),height=4,width=6)
par(mar=c(5.1, 4.1, 4.1, 2.1), tck=-0.04, cex.axis = 1.2, cex.lab=1.5)
par(mgp=c(2.5, 1, 0))
ylab.text = expression('R'['0'])
plot(xax_time,yax_r0,type="l",lwd=1.5, ylim=c(0.5,2.5), xlab="Week",ylab=ylab.text)
dev.off()
