library(deSolve)
library(ggplot2)
library(reshape2)
library(pracma)
library(grid)
library(gridExtra)
library(plotly)
library(gplots)
library(foreach)
library(doParallel)
library(abind)
library(RColorBrewer)

##########################################################################################
######################################Code for Plotting###################################
##########################################################################################
filled.contour3 <- function (x = seq(0, 1, length.out = nrow(z)),
                             y = seq(0, 1, length.out = ncol(z)), z, xlim = range(x, finite = TRUE), 
                             ylim = range(y, finite = TRUE), zlim = range(z, finite = TRUE), 
                             levels = pretty(zlim, nlevels), nlevels = 20, color.palette = cm.colors, 
                             col = color.palette(length(levels) - 1), plot.title, plot.axes, 
                             key.title, key.axes, asp = NA, xaxs = "i", yaxs = "i", las = 1, 
                             axes = TRUE, frame.plot = axes,mar, ...) 
{
  # modification by Ian Taylor of the filled.contour function
  # to remove the key and facilitate overplotting with contour()
  # further modified by Carey McGilliard and Bridget Ferris
  # to allow multiple plots on one page
  
  if (missing(z)) {
    if (!missing(x)) {
      if (is.list(x)) {
        z <- x$z
        y <- x$y
        x <- x$x
      }
      else {
        z <- x
        x <- seq.int(0, 1, length.out = nrow(z))
      }
    }
    else stop("no 'z' matrix specified")
  }
  else if (is.list(x)) {
    y <- x$y
    x <- x$x
  }
  if (any(diff(x) <= 0) || any(diff(y) <= 0)) 
    stop("increasing 'x' and 'y' values expected")
  # mar.orig <- (par.orig <- par(c("mar", "las", "mfrow")))$mar
  # on.exit(par(par.orig))
  # w <- (3 + mar.orig[2]) * par("csi") * 2.54
  # par(las = las)
  # mar <- mar.orig
  plot.new()
  # par(mar=mar)
  plot.window(xlim, ylim, "", xaxs = xaxs, yaxs = yaxs, asp = asp)
  if (!is.matrix(z) || nrow(z) <= 1 || ncol(z) <= 1) 
    stop("no proper 'z' matrix specified")
  if (!is.double(z)) 
    storage.mode(z) <- "double"
  .filled.contour(as.double(x), as.double(y), z, as.double(levels), 
                          col = col)
  if (missing(plot.axes)) {
    if (axes) {
      title(main = "", xlab = "", ylab = "")
      Axis(x, side = 1)
      Axis(y, side = 2)
    }
  }
  else plot.axes
  if (frame.plot) 
    box()
  if (missing(plot.title)) 
    title(...)
  else plot.title
  invisible()
}

filled.legend <- function (x = seq(0, 1, length.out = nrow(z)), y = seq(0, 1, length.out = ncol(z)), z, 
                           xlim = range(x, finite = TRUE), 
                           ylim = range(y, finite = TRUE), zlim = range(z, finite = TRUE), 
                           levels = pretty(zlim, nlevels), nlevels = 20, color.palette = cm.colors, 
                           col = color.palette(length(levels) - 1), plot.title, plot.axes, 
                           key.title, key.axes, asp = NA, xaxs = "i", yaxs = "i", las = 1, 
                           axes = TRUE, frame.plot = axes, ...) 
{
  # modification of filled.contour by Carey McGilliard and Bridget Ferris
  # designed to just plot the legend
  if (missing(z)) {
    if (!missing(x)) {
      if (is.list(x)) {
        z <- x$z
        y <- x$y
        x <- x$x
      }
      else {
        z <- x
        x <- seq.int(0, 1, length.out = nrow(z))
      }
    }
    else stop("no 'z' matrix specified")
  }
  else if (is.list(x)) {
    y <- x$y
    x <- x$x
  }
  if (any(diff(x) <= 0) || any(diff(y) <= 0)) 
    stop("increasing 'x' and 'y' values expected")
  #  mar.orig <- (par.orig <- par(c("mar", "las", "mfrow")))$mar
  #  on.exit(par(par.orig))
  #  w <- (3 + mar.orig[2L]) * par("csi") * 2.54
  #layout(matrix(c(2, 1), ncol = 2L), widths = c(1, lcm(w)))
  #  par(las = las)
  #  mar <- mar.orig
  #  mar[4L] <- mar[2L]
  #  mar[2L] <- 1
  #  par(mar = mar)
  # plot.new()
  plot.window(xlim = c(0, 1), ylim = range(levels), xaxs = "i", 
              yaxs = "i")
  rect(0, levels[-length(levels)], 1, levels[-1L], col = col)
  if (missing(key.axes)) {
    if (axes) 
      axis(4)
  }
  else key.axes
  box()
}

##########################################################################################
######################################ODE MODELS##########################################
##########################################################################################

############################################################
##Seasonality and Social Distancing and Vaccination
############################################################
sir_s_.model.seasonality.socdis.vax<-function (t,x,parameters) {
 
  SP=x[1]
  IP=x[2]
  R=x[3]
  SS=x[4]
  IS=x[5]
  V=x[6]
  
  N=parameters[["N"]]
  mu=parameters[["mu"]]
  delta=parameters[["delta"]]
  gamma=parameters[["gamma"]]
  R0.list=parameters[["R0.list"]]
  alpha=parameters[["alpha"]]
  epsilon=parameters[["epsilon"]]
  beta=R0.list[t]*gamma
  #Social distancing
  R0red=parameters[["R0red"]]
  tred1=parameters[["tred1"]]
  tred2=parameters[["tred2"]]
  tred3=parameters[["tred3"]]
  tred4=parameters[["tred4"]]
  #Vaccination
  nu=parameters[["nu"]]
  delta_vax=parameters[["delta_vax"]]
  tvax=parameters[["tvax"]]
  
  if(((t > tred1) & (t < tred2))|((t > tred3) & (t < tred4))){
    beta = beta * R0red
  }

  svax=0
  if(t>tvax){
    svax=1
  }
  
  #Equations
  dSPdt=mu*N-(beta*SP*(IP+alpha*IS)/N)-mu*SP-svax*nu*SP
  dIPdt=(beta*SP*(IP+alpha*IS)/N)-(gamma+mu)*IP
  dRdt=gamma*(IP+IS)-(delta+mu)*R
  dSSdt=delta*R-epsilon*(beta*SS*(IP+alpha*IS)/N)-mu*SS+delta_vax*V-svax*nu*SS
  dISdt=epsilon*(beta*SS*(IP+alpha*IS)/N)-(gamma+mu)*IS
  dVdt=svax*nu*(SP+SS)-delta_vax*V-mu*V
  
  dxdt <- c(dSPdt,dIPdt,dRdt,dSSdt,dISdt,dVdt)
  list(dxdt)
}

##########################################################################################
######################################Data input##########################################
##########################################################################################
NYdata=read.table("NY.csv",header=T,sep=",")
hku1=NYdata$hku1

##########################################################################################
######################################PLOTS###############################################
##########################################################################################

######################################
###########Run the ODEs###############
######################################

timenum = 52
gridsize = 50

N=1
mu=1/(50*52)
gamma=7/5

#Social distancing
##For Soc Dis on want to use R0red=0.6
R0red=0.6
tred1=15
tred2=56
tred3=0
tred4=0
#Seasonality
times<-seq(from=1,to=5*timenum,by=1)
R0.list <- rep(hku1,length=length(times))
##If you want to shift seasonal cycle do it here
seasonlag <- 0
R0.list <- circshift(R0.list,-seasonlag)

alpha=1
epsilon=0.7
ImmuneTime=1*timenum
delta=1/ImmuneTime

#Run the no vax scenario
parameters_novax<-list(N=N,mu=mu,gamma=gamma,alpha=alpha,
                       epsilon=epsilon,delta=delta, R0.list = R0.list, R0red=R0red,
                       tred1=tred1,tred2=tred2,tred3=tred3,tred4=tred4,
                       nu=1000,delta_vax=1000,tvax=1000*timenum)

times<-seq(from=1,to=timenum*5,by=1)
I0=1e-9
xstart<-c(SP=1-I0,IP=I0,R=0,SS=0,IS=0,V=0)
out_novax<-as.data.frame(rk(xstart,times, sir_s_.model.seasonality.socdis.vax,parameters_novax))

allIvax=function(nu,ImmuneTime_vax){
  
  delta_vax=1/ImmuneTime_vax
  
  parameters_vax<-list(N=N,mu=mu,gamma=gamma,alpha=alpha,
                    epsilon=epsilon,delta=delta, R0.list = R0.list, R0red=R0red,
                    tred1=tred1,tred2=tred2,tred3=tred3,tred4=tred4,
                    nu=nu,delta_vax=delta_vax,tvax=78)
  
  times<-seq(from=1,to=timenum*5,by=1)
  I0=1e-9
  xstart<-c(SP=1-I0,IP=I0,R=0,SS=0,IS=0,V=0) 

  out_vax<-as.data.frame(rk(xstart,times, sir_s_.model.seasonality.socdis.vax, parameters_vax))
  
  #Total
  Itotseries_vax <- out_vax$IP + out_vax$IS
  Itot_vax <- gamma * sum(Itotseries_vax[(79):(5*timenum)])
  Itotseries_novax <- out_novax$IP + out_novax$IS
  Itot_novax <- gamma * sum(Itotseries_novax[(79):(5*timenum)])
  
  #Primary
  Ipseries_vax <- out_vax$IP 
  Ip_vax <- gamma * sum(Ipseries_vax[(79):(5*timenum)])
  Ipseries_novax <- out_novax$IP 
  Ip_novax <- gamma * sum(Ipseries_novax[(79):(5*timenum)])
  
  #Secondary
  Isseries_vax <- out_vax$IS
  Is_vax <- gamma * sum(Isseries_vax[(79):(5*timenum)])
  Isseries_novax <- out_novax$IS
  Is_novax <- gamma * sum(Isseries_novax[(79):(5*timenum)])
  
  #plot(times,Itotseries_vax,type="l",lty=1,col="red")
  #lines(times,Itotseries_novax,lty=2,col="black")
  
  #plot(times,out_vax$V,type="l",lty=1,col="red",ylim=c(0,1))
  #lines(times,out_novax$V,type="l",lty=2,col="black")

  value <- list(Itot_vax/Itot_novax,Ip_vax/Ip_novax,Is_vax/Is_novax)
  
  return(value)
}

nu=seq(from=0,to=0.01,length.out=gridsize)
ImmuneTime_vax=seq(from=0.1*timenum,to=2*timenum,length.out=gridsize)

#nu=5e-3
#ImmuneTime_vax=0.5*timenum

out_totIVax_s <- matrix(0,length(nu),length(ImmuneTime_vax))
out_pIVax_s <- matrix(0,length(nu),length(ImmuneTime_vax))
out_sIVax_s <- matrix(0,length(nu),length(ImmuneTime_vax))

for(i in 1:length(nu)){
  for(j in 1:length(ImmuneTime_vax)){
    temp <- allIvax(nu[i],ImmuneTime_vax[j])
    out_totIVax_s[i,j] <- as.numeric(temp[[1]])
    out_pIVax_s[i,j] <- as.numeric(temp[[2]])
    out_sIVax_s[i,j] <- as.numeric(temp[[3]])
  }
}

##################################################
###########Plot with filled contour###############
##################################################

#####Total
rgb.palette <- colorRampPalette(brewer.pal(10,"GnBu"))(10)

plot.new()
pdf(sprintf('TotalIRatioVax_s_Rate_tvax1p5yr.pdf'),height=4,width=6)
par(mar=c(5.1, 4.1, 4.1, 2.1), tck=-0.02, cex.axis = 1 ,cex.main = 1.5)
par(mfrow=c(2,3), mai = c(0.4, 0.3, 0.4, 0.3))
par(mgp=c(2, 1, 0))
for (i in 1:1){
  
  filled.contour3(x=nu,y=ImmuneTime_vax/timenum,z=out_totIVax_s,
                  levels = pretty(c(0,1), 8),
                  col = rgb.palette,
                  plot.title=title(main = "",
                                   xlab="", 
                                   ylab="")
  )
}
par(new = "FALSE",plt = c(0.9,1.1,0.2,0.8),las = 1,cex.axis = 1)
filled.legend(x=nu,y=ImmuneTime_vax/timenum,z=matrix(0,length(nu),length(ImmuneTime_vax)),
              levels = pretty(c(0,1), 8), 
              col = rgb.palette,
              plot.title=title(main = "", xlab="", ylab=""))
dev.off()

######Primary
rgb.palette <- colorRampPalette(brewer.pal(10,"GnBu"))(10)

plot.new()
pdf(sprintf('PrimaryIRatioVax_s_Rate_tvax1p5yr.pdf'),height=4,width=6)
par(mar=c(5.1, 4.1, 4.1, 2.1), tck=-0.02, cex.axis = 1 ,cex.main = 1.5)
par(mfrow=c(2,3), mai = c(0.4, 0.3, 0.4, 0.3))
par(mgp=c(2, 1, 0))
for (i in 1:1){
  
  filled.contour3(x=nu,y=ImmuneTime_vax/timenum,z=out_pIVax_s,
                  levels = pretty(c(0,1), 8), 
                  col = rgb.palette,
                  plot.title=title(main = "",
                                   xlab="", 
                                   ylab="")
  )
}
par(new = "FALSE",plt = c(0.9,1.1,0.2,0.8),las = 1,cex.axis = 1)
filled.legend(x=nu,y=ImmuneTime_vax/timenum,z=matrix(0,length(nu),length(ImmuneTime_vax)),
              levels = pretty(c(0,1), 8), 
              col = rgb.palette,
              plot.title=title(main = "", xlab="", ylab=""))
dev.off()

#####Secondary
rgb.palette <- colorRampPalette(brewer.pal(11,"GnBu"))(11)

plot.new()
pdf(sprintf('SecondaryIRatioVax_s_Rate_tvax1p5yr.pdf'),height=4,width=6)
par(mar=c(5.1, 4.1, 4.1, 2.1), tck=-0.02, cex.axis = 1 ,cex.main = 1.5)
par(mfrow=c(2,3), mai = c(0.4, 0.3, 0.4, 0.3))
par(mgp=c(2, 1, 0))
for (i in 1:1){
  
  filled.contour3(x=nu,y=ImmuneTime_vax/timenum,z=out_sIVax_s,
                  levels = pretty(c(0,1.1), 8), 
                  col = rgb.palette,
                  plot.title=title(main = "",
                                   xlab="", 
                                   ylab="")
  )
}
par(new = "FALSE",plt = c(0.9,1.1,0.2,0.8),las = 1,cex.axis = 1)
filled.legend(x=nu,y=ImmuneTime_vax/timenum,z=matrix(0,length(nu),length(ImmuneTime_vax)),
              levels = pretty(c(0,1.1), 8), 
              col = rgb.palette,
              plot.title=title(main = "", xlab="", ylab=""))
dev.off()

